from logging import Logger
from typing import Any

from .functions import get_full_class_name


class BaseLoggingService:
    def __init__(self, logger: Logger = None):
        self.logger = logger

    def set_logger(self, logger: Logger):
        self.logger = logger

    def logging_error(self, exception: Any, message: str):
        self.logger.error(message)
        self.logger.error(f"{get_full_class_name(exception)}: {str(exception)}")


from .base_logging_service import *
from .functions import *
from .redis_helper import *
from .repository_manager_helper import *
from .revolt_helper import *
from .session_manager_helper import *
from .proxy_manager_helper import *
from .proxy_manager_helper.proxy_manager_helper import *

__all__ = [
    #Classes
    "BaseLoggingService",
    "ProxySchema",
    "ProxyManagerHelper",
    "RepositoryManagerHelper",
    "RevoltHelper",
    "SessionManagerHelper",
    #Functions
    "get_full_class_name",
    "timestamp",
    "get_moscow_datetime",
    "get_moscow_date",
    "current_data_add",
    "get_logger",
    "digits_only"
]


import logging
import aiohttp


class DiscordHelper:
    def __init__(self, url):
        self.url = url
        self.title = None
        self.description = None
        self.color = None
        self.notify_everyone = False

    def set_title(self, title):
        self.title = title

    def set_description(self, description):
        self.description = description

    def set_colour(self, colour):
        self.color = colour

    def set_notify_everyone(self, notify_everyone):
        self.notify_everyone = notify_everyone

    async def send_error(self, error, desc: str = None):
        self.title = "[Error]"
        self.description = desc
        self.color = "16711680"
        await self._send_message(f"```{error}```")

    async def send_warning(self, warning, desc: str = None):
        self.title = "[Warning]"
        self.description = desc
        self.color = "16746496"
        await self._send_message(f"```{warning}```")

    async def send_info(self, info, desc: str = None):
        self.title = "[Info]"
        self.description = desc
        self.color = "65280"
        await self._send_message(f"```{info}```")

    async def _send_message(self, message: str):
        try:
            async with aiohttp.ClientSession() as session:
                response = await session.post(
                    self.url,
                    json={
                        "content": f"{'@everyone 'if self.notify_everyone else ''}{message}",
                        "tts": False,
                        "username": "Система уведомлений",
                        "embeds": [
                            {
                                "title": self.title,
                                "description": self.description,
                                "color": self.color
                            }
                        ],
                    },
                )
                if response.status != 204:
                    logging.warn(
                        f"Non-204 response from Discord! Response: {await response.text()}"
                    )
        except Exception as e:
            logging.error(f"Could not send message to Discord: {e}")



from sqlalchemy.ext.asyncio import AsyncSession


class RepositoryManagerHelper:
    def __init__(self, session: AsyncSession):
        self.session = session

    def get_session(self):
        return self.session

    def close_session(self):
        return self.session

    async def __aenter__(self):
        pass

    async def session_close(self):
        await self.session.close()



import aiohttp

from .schemas.proxy_schema import ProxySchema


class ProxyManagerHelper:
    def __init__(self, proxy_manager_url: str):
        self.proxy_manager_url = proxy_manager_url

    async def get_one_proxy(self, queue: str) -> ProxySchema:
        async with aiohttp.ClientSession() as session:
            async with session.get(
                self.proxy_manager_url + "/proxies", params={"queue": queue}
            ) as response:
                proxy_text = await response.text()
                return ProxySchema.model_validate_json(proxy_text)



import logging
import aiohttp


class RevoltHelper:
    def __init__(self, url, token, channel_id):
        self.message_url = f"{url}/channels/{channel_id}/messages"
        self.token = token
        self.title = None
        self.description = None
        self.colour = None

    def set_title(self, title):
        self.title = title

    def set_description(self, description):
        self.description = description

    def set_colour(self, colour):
        self.colour = colour

    async def send_error(self, error, desc: str = None):
        self.title = "[Error]"
        self.description = desc
        self.colour = "#ff0000"
        await self._send_message(f"```{error}```")

    async def send_warning(self, warning, desc: str = None):
        self.title = "[Warning]"
        self.description = desc
        self.colour = "#ff8800"
        await self._send_message(f"```{warning}```")

    async def send_info(self, info, desc: str = None):
        self.title = "[Info]"
        self.description = desc
        self.colour = "#00ff00"
        await self._send_message(f"```{info}```")

    async def _send_message(self, message: str):
        try:
            async with aiohttp.ClientSession() as session:
                response = await session.post(
                    self.message_url,
                    json={
                        "content": message,
                        "embeds": [
                            {
                                "title": self.title,
                                "description": self.description,
                                "colour": self.colour,
                            }
                        ],
                    },
                    headers={
                        "X-Bot-Token": self.token,
                        "Content-Type": "application/json",
                    },
                )
                if response.status != 200:
                    logging.warn(
                        f"Non-200 response from Revolt! Response: {await response.text()}"
                    )
        except Exception as e:
            logging.error(f"Could not send message to Revolt: {e}")
